from django.db import models
from django.utils.translation import gettext_lazy as _
from artd_partner.models import Partner
from artd_customer.models import Customer

class Base(models.Model):
    created_at = models.DateTimeField(
        auto_now_add=True,
    )
    updated_at = models.DateTimeField(
        auto_now=True,
    )
    status = models.BooleanField(
        "status",
        default=True,
    )

    class Meta:
        abstract = True

class OpenPay(Base):
    """Model definition for MercadoPago."""

    partner = models.ForeignKey(
        Partner,
        help_text=_("Partner"),
        on_delete=models.CASCADE,
        
    )

    production_client_id = models.CharField(
        _("Production client id"),
        help_text=_("Production client id"),
        max_length=250,
    )
    production_client_secret = models.CharField(
        _("Production client secret"), 
        help_text=_("Production client secret"),
        max_length=250,
    )
    production_public_key = models.CharField(
        _("Production public key"),
        help_text=_("Production public key"),
        max_length=250,
    )

    class Meta:
        """Meta definition for OpenPay."""

        verbose_name = "Open Pay"
        verbose_name_plural = "Open Pay"

    def __str__(self):
        return str(self.id)
        """Unicode representation of OpenPay."""

class OpenPayCard(Base):
    customer = models.ForeignKey(
        Customer,
        help_text=_("Customer"),
        on_delete=models.CASCADE,
    )

    bank_code = models.CharField(
        _("Bank Code"),
        help_text=_("Bank Code"),
        max_length=250,
    )
    bank_name = models.CharField(
        _("Bank Name"),
        help_text=_("Bank Name"),
        max_length=250,
    )
    brand = models.CharField(
        _("Brand"),
        help_text=_("Brand"),
        max_length=250,
    )
    card_number = models.CharField(
        _("Card Number"),
        help_text=_("Card Number"),
        max_length=250,
    )
    expiration_month = models.CharField(
        _("Expiration Month"),
        help_text=_("Expiration Month"),
        max_length=250,
    )
    expiration_year = models.CharField(
        _("Expiration Year"),
        help_text=_("Expiration Year"),
        max_length=250,
    )
    holder_name = models.CharField(
        _("Holder Name"),
        help_text=_("Holder Name"),
        max_length=250,
    )
    card_id = models.CharField(
        _("Card Id"),
        help_text=_("Card Id"),
        max_length=250,
    )
    card_type = models.CharField(
        _("Card Type"),
        help_text=_("Card Type"),
        max_length=250,
    )
    other_data = models.JSONField(
        help_text=_("Card Type"),
        null = True,
        blank = True,
        default = dict
    )

    class Meta:
        """Meta definition for OpenPay."""

        verbose_name = "Open Pay Card"
        verbose_name_plural = "Open Pay Cards"

    def __str__(self):
        return str(self.card_id)
        """Unicode representation of OpenPay."""

class OpenpayCharge(Base):

    card = models.ForeignKey(
        OpenPayCard,
        help_text=_("Card Id"),
        on_delete=models.CASCADE,
    )
    amount = models.FloatField(
        _("Charge Amount"),
        help_text=_("Charge Amounts"),
        default = 0
    )
    charge_id = models.CharField(
        _("Charge id"),
        help_text=_("Charge id"),
        max_length=250,
    )
    authorization = models.CharField(
        _("Charge Authorization"),
        help_text=_("Charge Authorizations"),
        max_length=250,
    )
    status = models.CharField(
        _("Charge Status"),
        help_text=_("Charge Status"),
        max_length=250,
    )
    other_data = models.JSONField(
        help_text=_("Card Charge"),
        null = True,
        blank = True,
        default = dict
    )
    class Meta:
        """Meta definition for OpenPay."""

        verbose_name = "Open Pay Charge"
        verbose_name_plural = "Open Pay Charges"

    def __str__(self):
        return str(self.charge_id)
        """Unicode representation of OpenPay."""







# Generated by Django 5.0.2 on 2024-02-23 21:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("artd_openpay", "0005_openpaycharge"),
    ]

    operations = [
        migrations.AlterField(
            model_name="openpaycharge",
            name="charge_id",
            field=models.CharField(
                help_text="Charge id", max_length=250, verbose_name="Charge id"
            ),
        ),
    ]



import openpay
from artd_openpay.models import OpenPay , OpenPayCard , OpenpayCharge
from artd_partner.models import Partner
from artd_customer.models import Customer
class OpenPayUtil():
    
    __partner=None
    __production_client_id=None
    __production_client_secret=None
    __production_public_key=None

    def __init__(self, partner:Partner):
        self.__partner = OpenPay.objects.filter(partner=partner).first()
        self.__production_client_id = self.__partner.production_client_id
        self.__production_client_secret = self.__partner.production_client_secret
        self.__production_public_key = self.__partner.production_public_key
        openpay.api_key = self.__production_client_secret
        openpay.verify_ssl_certs = False
        openpay.merchant_id = self.__production_client_id
        openpay.country = 'co'
        print(self.__production_client_id , self.__production_client_secret , self.__production_public_key,sep="\n")
    
    def create_customer(self,name,last_name,email,phone_number):
        try:
            customer_count = Customer.objects.filter(email=email , phone = phone_number).count()
            if customer_count == 0:
                customer = openpay.Customer.create(
                    name=name,
                    last_name=last_name,
                    email=email,
                    phone_number=phone_number
                )
                Customer.objects.create(
                    name = name,
                    last_name = last_name,
                    email = email,
                    phone = phone_number,
                    other_data  = customer,
                )
                print(customer)
            else:
                print("The customer already exists")
        except Exception as e:
            print(e)

    def get_openpay_customer(self,email,phone):
        artd_customer=Customer.objects.filter(email=email,phone=phone).first()
        print(artd_customer)
        other_data = artd_customer.other_data
        print(other_data["id"])
        customer = openpay.Customer.retrieve(other_data["id"])
        return customer , artd_customer

    def store_card(self,email,phone,card_number,holder_name,expiration_year,expiration_month,cvv2):
        customer , artd_customer = self.get_openpay_customer(email=email,phone=phone)
        print(customer)
        card = customer.cards.create(
            card_number=card_number,
            holder_name=holder_name,
            expiration_year=expiration_year,
            expiration_month=expiration_month,
            cvv2=cvv2,
        )
        OpenPayCard.objects.create(
            customer = artd_customer,
            bank_code = card["bank_code"],
            brand = card["brand"],
            card_number = card["card_number"],
            expiration_month = card["expiration_month"],
            expiration_year = card["expiration_year"],
            holder_name = card["holder_name"],
            card_id = card["id"],
            card_type = card["type"],
            other_data = card
        )

    def delete_card(self,email,phone,card_id):
        try:
            print(email,phone,sep="\n")
            customer , artd_customer = self.get_openpay_customer(email=email,phone=phone)
            card = customer.cards.retrieve(card_id)
            card.delete()
        except Exception as e:
            print(e)
        try:
            OpenPayCard.objects.filter(card_id=card_id).delete()
        except Exception as e:
            print(e)

    def create_charge(self, email ,phone , card_id ,device_session_id,amount ):
        customer , artd_customer = self.get_openpay_customer(email=email,phone=phone)
        card = OpenPayCard.objects.filter(card_id=card_id).first()
        charge = customer.charges.create(
            source_id=card_id, 
            method="card", 
            amount=amount, 
            description="Charge", 
            capture=False, 
            currency = "COP" , 
            device_session_id = device_session_id
        )
        charge.capture(merchant=False)
        OpenpayCharge.objects.create(
            card=card,
            amount=charge["amount"],
            charge_id=charge["id"],
            authorization=charge["authorization"],
            status=charge["status"],
            other_data = charge
        )
        print(charge)

    def all_customers_card(self,email,phone):
        artd_customer=Customer.objects.filter(email=email,phone=phone).first()
        customer_open_pay_id = artd_customer.other_data["id"]
        customer = openpay.Customer.retrieve(customer_open_pay_id)
        cards = customer.cards.all()
        print(cards)


# Generated by Django 5.0.2 on 2024-02-23 21:26

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("artd_openpay", "0004_alter_openpaycard_other_data"),
    ]

    operations = [
        migrations.CreateModel(
            name="OpenpayCharge",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "amount",
                    models.FloatField(
                        default=0,
                        help_text="Charge Amounts",
                        verbose_name="Charge Amount",
                    ),
                ),
                (
                    "charge_id",
                    models.FloatField(
                        default=0, help_text="Charge id", verbose_name="Charge id"
                    ),
                ),
                (
                    "authorization",
                    models.CharField(
                        help_text="Charge Authorizations",
                        max_length=250,
                        verbose_name="Charge Authorization",
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        help_text="Charge Status",
                        max_length=250,
                        verbose_name="Charge Status",
                    ),
                ),
                (
                    "other_data",
                    models.JSONField(
                        blank=True, default=dict, help_text="Card Charge", null=True
                    ),
                ),
                (
                    "card",
                    models.ForeignKey(
                        help_text="Card Id",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="artd_openpay.openpaycard",
                    ),
                ),
            ],
            options={
                "verbose_name": "Open Pay Charge",
                "verbose_name_plural": "Open Pay Charges",
            },
        ),
    ]



# Generated by Django 5.0.2 on 2024-02-23 15:31

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("artd_partner", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="OpenPay",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("status", models.BooleanField(default=True, verbose_name="status")),
                (
                    "production_client_id",
                    models.CharField(
                        help_text="Production client id",
                        max_length=250,
                        verbose_name="Production client id",
                    ),
                ),
                (
                    "production_client_secret",
                    models.CharField(
                        help_text="Production client secret",
                        max_length=250,
                        verbose_name="Production client secret",
                    ),
                ),
                (
                    "production_public_key",
                    models.CharField(
                        help_text="Production public key",
                        max_length=250,
                        verbose_name="Production public key",
                    ),
                ),
                (
                    "Partner",
                    models.ForeignKey(
                        help_text="Partner",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="artd_partner.partner",
                    ),
                ),
            ],
            options={
                "verbose_name": "OpenPay",
                "verbose_name_plural": "OpenPay",
            },
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _



class ArtdOpenpayConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "artd_openpay"
    verbose_name = _("Artd Open Pay")



# Generated by Django 5.0.2 on 2024-02-23 20:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("artd_openpay", "0003_alter_openpay_options_openpaycard"),
    ]

    operations = [
        migrations.AlterField(
            model_name="openpaycard",
            name="other_data",
            field=models.JSONField(
                blank=True, default=dict, help_text="Card Type", null=True
            ),
        ),
    ]



from django.contrib import admin
from artd_openpay.models import OpenPay , OpenPayCard , OpenpayCharge
from django_json_widget.widgets import JSONEditorWidget
from django.db import models


# Register your models here.
@admin.register(OpenPay)
class OpenPayAdmin(admin.ModelAdmin):
    list_display = (
        "production_client_id",
        "status",
        "created_at",
        "updated_at",
    )
    list_filter = (
        "status",
        "created_at",
        "updated_at",
    )
    search_fields = (
        "production_client_id",
        "status",
        "created_at",
        "updated_at",
    )
    readonly_fields = (
        "created_at",
        "updated_at",
    )

@admin.register(OpenPayCard)
class OpenPayCardAdmin(admin.ModelAdmin):
    list_display = (
        "customer",
        "card_id",
        "brand",
        "card_type",
    )
    list_filter = (
        "status",
        "created_at",
        "updated_at",
    )
    search_fields = (
        "customer__email",
        "customer__phone",
        "card_id",
        "brand",
        "card_type",
    )
    readonly_fields = (
        "customer",
        "card_id",
        "brand",
        "card_type",
        "bank_code",
        "bank_name",
        "expiration_month",
        "expiration_year",
        "holder_name",
        "card_number",
    )
    formfield_overrides = {
        models.JSONField: {"widget": JSONEditorWidget},
    }

@admin.register(OpenpayCharge)
class OpenPayChargedAdmin(admin.ModelAdmin):
    list_display = (
        "amount",
        "charge_id",
        "authorization",
        "status"
    )
    list_filter = (
        "status",
        "created_at",
        "updated_at",
    )
    search_fields = (
        "amount",
        "charge_id",
        "authorization",
        "status"
    )
    readonly_fields = (
        "card",
        "amount",
        "charge_id",
        "authorization",
        "status"
    )
    formfield_overrides = {
        models.JSONField: {"widget": JSONEditorWidget},
    }


# Generated by Django 5.0.2 on 2024-02-23 16:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("artd_openpay", "0001_initial"),
    ]

    operations = [
        migrations.RenameField(
            model_name="openpay",
            old_name="Partner",
            new_name="partner",
        ),
    ]



# Generated by Django 5.0.2 on 2024-02-23 19:53

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("artd_customer", "0003_customer_external_id_customer_source_and_more"),
        ("artd_openpay", "0002_rename_partner_openpay_partner"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="openpay",
            options={"verbose_name": "Open Pay", "verbose_name_plural": "Open Pay"},
        ),
        migrations.CreateModel(
            name="OpenPayCard",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("status", models.BooleanField(default=True, verbose_name="status")),
                (
                    "bank_code",
                    models.CharField(
                        help_text="Bank Code", max_length=250, verbose_name="Bank Code"
                    ),
                ),
                (
                    "bank_name",
                    models.CharField(
                        help_text="Bank Name", max_length=250, verbose_name="Bank Name"
                    ),
                ),
                (
                    "brand",
                    models.CharField(
                        help_text="Brand", max_length=250, verbose_name="Brand"
                    ),
                ),
                (
                    "card_number",
                    models.CharField(
                        help_text="Card Number",
                        max_length=250,
                        verbose_name="Card Number",
                    ),
                ),
                (
                    "expiration_month",
                    models.CharField(
                        help_text="Expiration Month",
                        max_length=250,
                        verbose_name="Expiration Month",
                    ),
                ),
                (
                    "expiration_year",
                    models.CharField(
                        help_text="Expiration Year",
                        max_length=250,
                        verbose_name="Expiration Year",
                    ),
                ),
                (
                    "holder_name",
                    models.CharField(
                        help_text="Holder Name",
                        max_length=250,
                        verbose_name="Holder Name",
                    ),
                ),
                (
                    "card_id",
                    models.CharField(
                        help_text="Card Id", max_length=250, verbose_name="Card Id"
                    ),
                ),
                (
                    "card_type",
                    models.CharField(
                        help_text="Card Type", max_length=250, verbose_name="Card Type"
                    ),
                ),
                (
                    "other_data",
                    models.JSONField(
                        blank=True, default={}, help_text="Card Type", null=True
                    ),
                ),
                (
                    "customer",
                    models.ForeignKey(
                        help_text="Customer",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="artd_customer.customer",
                    ),
                ),
            ],
            options={
                "verbose_name": "Open Pay Card",
                "verbose_name_plural": "Open Pay Cards",
            },
        ),
    ]
